@@ -7,37 +7,31 @@
 
 module Command.FromKey where
 
-import Control.Monad.State (liftIO)
-import System.Posix.Files
-import System.Directory
-import Control.Monad (unless)
-
+import Common.Annex
 import Command
-import qualified AnnexQueue
-import Utility
-import Content
-import Messages
+import qualified Annex.Queue
+import Annex.Content
 import Types.Key
 
-command :: [Command]
-command = [repoCommand "fromkey" paramPath seek
+def :: [Command]
+def = [command "fromkey" (paramPair paramKey paramPath) seek
 	"adds a file using a specific key"]
 
 seek :: [CommandSeek]
-seek = [withFilesMissing start]
+seek = [withWords start]
 
-start :: CommandStartString
-start file = notBareRepo $ do
-	key <- cmdlineKey
+start :: [String] -> CommandStart
+start (keyname:file:[]) = notBareRepo $ do
+	let key = fromMaybe (error "bad key") $ readKey keyname
 	inbackend <- inAnnex key
 	unless inbackend $ error $
-		"key ("++keyName key++") is not present in backend"
+		"key ("++ keyname ++") is not present in backend"
 	showStart "fromkey" file
-	next $ perform file
+	next $ perform key file
+start _ = error "specify a key and a dest file"
 
-perform :: FilePath -> CommandPerform
-perform file = do
-	key <- cmdlineKey
+perform :: Key -> FilePath -> CommandPerform
+perform key file = do
 	link <- calcGitLink file key
 	liftIO $ createDirectoryIfMissing True (parentDir file)
 	liftIO $ createSymbolicLink link file
@@ -45,5 +39,5 @@ perform file = do
 
 cleanup :: FilePath -> CommandCleanup
 cleanup file = do
-	AnnexQueue.add "add" [Param "--"] [file]
+	Annex.Queue.add "add" [Param "--"] [file]
 	return True
